//=============================================================================
// file: ImpressionistModel.h
// desc:
//		 Implements the model part of the standard MVC (model-view-controller)
//		 pattern.  This class is meant to be independent of both the window
//		 system and the user input.  It is merely a repository for the state
//		 of the application, in this case:
//			* original image bitmap
//			* impressionist image bitmap
//			* current brush size
//			* current brush
//		 Note that the brushes themselves are stored in the ImpressionistView
//		 instance, as they are specific to the window system being used
//		 (OpenGL in this case).
//=============================================================================

#ifndef __IMPRESSIONIST_MODEL_HEADER__
#define __IMPRESSIONIST_MODEL_HEADER__

#include "impressionist.h"
#include "bitmap.h"
#include "AbstractBrush.h"

//==========[ class ImpressionistModel ]=======================================

class ImpressionistModel {

private:

	//---[ Image and Bitmap Variables ]--------------------

		// bitmap of original, unaltered image
	unsigned char*	mOriginalBitmap;
		// bitmap of the impressionist image
	unsigned char*	mPaintedBitmap;
		// bitmap size
	int				mBitmapWidth;
	int				mBitmapHeight;

	//---[ Brush Variables ]-------------------------------

		// current brush size
	int				mBrushSize;
		// current brush
	AbstractBrush*	mCurrentBrush;

public:

	//---[ Constructor ]-----------------------------------

	ImpressionistModel();

	//---[ Image Methods ]---------------------------------

		// get the bitmap dimentions
	int getBitmapWidth() const { return mBitmapWidth; }
	int getBitmapHeight() const { return mBitmapHeight; }

		// get pointers to the start of the bitmaps
	unsigned char* getOriginalBitmap() const { return mOriginalBitmap; }
	unsigned char* getPaintedBitmap() const { return mPaintedBitmap; }

		// returns true if an image is currently loaded
	bool imageLoaded() const { return mOriginalBitmap != NULL; }
	
		// load an image from a file
	int loadImage( char *iname );
		// save the current impressionist image to a file
	int saveImage( char *fname ) const;

		// return the value of the pixel at (x,y)
	unsigned char* getOriginalPixel( int x, int y );
	unsigned char* getOriginalPixel( const Point p );

	//---[ Brush Methods ]---------------------------------

		// returns the current brush size
	int getBrushSize() const { return mBrushSize; }
		// sets the current brush size
	void setBrushSize( int size ) { mBrushSize = size; }

		// get and set the current brush (note that the brushes
		// are stored in an instance of ImpressionistView)
	void setCurrentBrush( AbstractBrush* brush ) { mCurrentBrush = brush; }
	AbstractBrush* getCurrentBrush() const { return mCurrentBrush; }
};

#endif
